@@ -14,23 +14,25 @@
 
 #ifdef R_MESON_VERSION
 #define R2_PREFIX "/usr"
+#define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/5.1.1/www"
-#define R2_PLUGINS "lib/radare2/5.1.1"
-#define R2_EXTRAS "lib/radare2-extras/5.1.1"
-#define R2_BINDINGS "lib/radare2-bindings/5.1.1"
+#define R2_WWWROOT "/usr/share/radare2/5.2.0/www"
+#define R2_PLUGINS "lib/radare2/5.2.0"
+#define R2_EXTRAS "lib/radare2-extras/5.2.0"
+#define R2_BINDINGS "lib/radare2-bindings/5.2.0"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/5.1.1"
-#define R2_ZIGNS "share/radare2/5.1.1/zigns"
-#define R2_THEMES "share/radare2/5.1.1/cons"
+#define R2_SDB "share/radare2/5.2.0"
+#define R2_ZIGNS "share/radare2/5.2.0/zigns"
+#define R2_THEMES "share/radare2/5.2.0/cons"
 #define R2_FORTUNES "share/doc/radare2"
-#define R2_FLAGS "share/radare2/5.1.1/flag"
-#define R2_HUD "share/radare2/5.1.1/hud"
+#define R2_FLAGS "share/radare2/5.2.0/flag"
+#define R2_HUD "share/radare2/5.2.0/hud"
 #else
 #if __WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
+#define R2_ETCDIR "etc"
 #define R2_LIBDIR "lib"
 #define R2_INCDIR "include\\libr"
 #define R2_DATDIR "share"
@@ -40,6 +42,7 @@
 #define R2_BINDINGS "lib\\bindings"
 #else
 #define R2_PREFIX "/usr"
+#define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr/libr"
 #define R2_DATDIR "/usr/share"
@@ -90,7 +93,7 @@
 #define USE_LIB_XXHASH 1
 
 #ifndef HAVE_LIB_SSL
-#define HAVE_LIB_SSL 1
+#define HAVE_LIB_SSL 0
 #endif
 
 #define HAVE_LIBUV 1
