@@ -30,8 +30,10 @@ typedef struct r_charset_rune_t {
 
 typedef struct r_charset_t {
 	Sdb *db;
+	Sdb *db_char_to_hex;
 	RCharsetRune *custom_charset;
-	size_t remaining;
+	size_t encode_maxkeylen;
+	size_t decode_maxkeylen;
 } RCharset;
 
 #define R_STR_ISEMPTY(x) (!(x) || !*(x))
@@ -42,7 +44,8 @@ R_API RCharset *r_charset_new(void);
 R_API void r_charset_free(RCharset *charset);
 R_API RCharsetRune *r_charset_rune_new(const ut8 *ch, const ut8 *hx);
 R_API void r_charset_rune_free(RCharsetRune *rcr);
-R_API size_t r_charset_encode_str(RCharset *r_char, ut8 *out, size_t out_len, const ut8 *in, size_t len_input);
+R_API size_t r_charset_encode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
+R_API size_t r_charset_decode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
 R_API bool r_charset_open(RCharset *c, const char *cs);
 R_API RCharsetRune * add_rune(RCharsetRune *rcsr, const ut8 *ch, const ut8 *hx);
 R_API RCharsetRune *search_from_hex(RCharsetRune *rcsr, const ut8 *hx);
@@ -59,6 +62,7 @@ R_API int r_str_bounds(const char *str, int *h);
 R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
 R_API char *r_str_scale(const char *r, int w, int h);
 R_API bool r_str_range_in(const char *r, ut64 addr);
+R_API int r_str_size(const char *s, int *rows);
 R_API size_t r_str_len_utf8(const char *s);
 R_API size_t r_str_len_utf8_ansi(const char *str);
 R_API size_t r_str_len_utf8char(const char *s, int left);
@@ -107,6 +111,7 @@ R_API bool r_str_is_printable_limited(const char *str, int size);
 R_API bool r_str_is_printable_incl_newlines(const char *str);
 R_API char *r_str_appendlen(char *ptr, const char *string, int slen);
 R_API char *r_str_newf(const char *fmt, ...) R_PRINTF_CHECK(1, 2);
+R_API int r_str_distance(const char *a, const char *b);
 R_API char *r_str_newlen(const char *str, int len);
 R_API const char *r_str_sysbits(const int v);
 R_API char *r_str_trunc_ellipsis(const char *str, int len);
@@ -177,6 +182,7 @@ R_API int r_str_unescape(char *buf);
 R_API char *r_str_sanitize_r2(const char *buf);
 R_API char *r_str_escape(const char *buf);
 R_API char *r_str_escape_sh(const char *buf);
+R_API char *r_str_escape_sql(const char *buf);
 R_API char *r_str_escape_dot(const char *buf);
 R_API char *r_str_escape_latin1(const char *buf, bool show_asciidot, bool esc_bslash, bool colors);
 R_API char *r_str_escape_utf8(const char *buf, bool show_asciidot, bool esc_bslash);
